#!/bin/bash
# (c) Copyright 2007-2013 Hewlett Packard Development Company, L.P.
# Version A.12.00.00

################################################################################
# HA Apache Toolkit Main Script
################################################################################
# This script contains a list of internal-use variables and functions that 
# support start/stop of an Apache instance and its monitor process. To be a part
# of a ServiceGuard package, this script has to be placed in the package 
# directory. This script can be called by the Toolkit Interface Script or 
# directly by the Package Control Script.
#
# Parameters:
#    Action:    either "start" or "stop" or "monitor"
#  
# Exit Code:
#   1 - when a failure occurs during the "start" or "stop" or "monitor" process.
#   0 - normal exit.
#
# REVISION HISTORY:
#   Version      mm/dd/yyyy   Changes
#   A.03.00      07/13/2006   1) Removed launch_monitor script and halt_monitor
#                                script as Post-IP concept for Apache toolkit is 
#                                removed. The monitor service details should be  
#                                added in package control script.
#            	              2) Modified the script to handle new argument 
#                                "monitor".
#                             3) Added code to find whether httpd or httpd2 is an 
#		    	         apache server process.
#                             4) Fixed the bug in stop_server that was not halting 
#			         all the Apache processes properly.
#                             5) Added code to remove the stale apache pid file,if 
#			         present, before starting or after halting Apache
#			         processes.				
#                             6) Added code related to maintenance mode
#                             7) Added code to display warning message if SELinux
#			         is enabled.
#                             8) Removed -d option and added -f option while 
#                                calling httpd process. This is because, on SLES,
#                               -d option alone takes the default httpd.conf
#			         instead of httpd.conf present in SERVER_ROOT
#			         directory.
#                             9) Removed get_moduleargs function
#
#   A.04.00      03/25/2008   1) Modular package support
#  
#   A.04.02.01   04/30/2012   1) Added check for the attributes FAIL_CRITERIA,
#                                MONITOR_INTERVAL.
#
#   A.04.02.02   10/19/2012   1) Changed default value of RETRY_TIMES to 0.
#                             2) Changed 'kill -15' command to 'httpd -k stop'
#                                command. 
#   A.12.00.00   06/06/2013   1) Removed Toolkit Maintenance & Legacy package support 
################################################################################

# Note: Do not modify this file

. /etc/cmcluster.conf

PATH=/bin:/sbin:/usr/bin:/usr/sbin:/bin:$SGSBIN:

APACHE_TIMEOUT=5
CONF_DIR=/tmp

if [ -z $HTTPD_CONFIG_FILE ]; then
    echo "ERROR: The HTTPD_CONFIG_FILE value has not been defined."
    exit 1
fi

if [ -z $FAIL_CRITERIA ]; then
    FAIL_CRITERIA="Parent_Process_Only"	
fi

# Assigning default value 30 to MONITOR_INTERVAL.
if [ -z $MONITOR_INTERVAL ]; then
    MONITOR_INTERVAL=30 
fi

if [ ! -f $HTTPD_CONFIG_FILE ]; then
   echo "ERROR: The configuration file $HTTPD_CONFIG_FILE does not exist."
   exit 1
fi

# Make sure that RETRY_TIMES has been defined otherwise, use the toolkit
# default
RETRY_TIMES=${RETRY_TIMES:=0}

# Make sure there is a PID file
if [ -z "$PID_FILE" ] ; then
    PID_FILE="$CONF_DIR/$SG_PACKAGE.pid"
fi

FAIL_CRITERIA=${FAIL_CRITERIA:=Parent_Process_Only}

#Check whether httpd or httpd2 is Apache server process
if [ -f /usr/sbin/httpd ]; then
   StartupCmd=/usr/sbin/httpd
   HTTP_SERVER_PROC=httpd
elif [ -f /usr/sbin/httpd2 ]; then
   StartupCmd=/usr/sbin/httpd2
   HTTP_SERVER_PROC=httpd2
fi


# This function starts the HTTP daemons
#
function start_server
{
    echo "$(date '+%b %e %T') - Node \"$(hostname)\": Starting Apache daemons."
    if [ -f $PID_FILE ] ; then
       echo "$(date '+%b %e %T')- Node \"$(hostname)\": Warning: Pid File already present"
       # remove the pid file if no httpd process is running
       remove_pidfile
    fi

    #Checking for selinux enabled or disabled 
    selinuxenabled >/dev/null 2>&1
    IS_SE_ENABLED=`echo $?`

    # If SELinux is enabled then warn the user
    if [ $IS_SE_ENABLED -eq 0 ]
    then
        echo "$(date '+%b %e %T') - Node \"$(hostname)\": Warning: SELinux is enabled"
    fi
    $StartupCmd -f $HTTPD_CONFIG_FILE -c "PidFile $PID_FILE" 
    ret_value=$?
    check_return $ret_value 1

    # Check if the pid file is ready.
    counter=0    
    while :; do
        if [ -f $PID_FILE ]; then
            break
        else
            if [ $counter -ge 10 ]; then
                echo "ERROR: Function start_server: httpd failed to create PID file $PID_FILE"
                check_return 1 1
            else
                sleep 1
                let counter="$counter+1"
            fi
        fi
    done
}

# This function removes pid file if no http daemon is running
function remove_pidfile
{
   if [ -f $PID_FILE ] ; then
      read PID < $PID_FILE
      rpid=`ps $PID 2>/dev/null | grep $HTTP_SERVER_PROC | awk '{print $1}'`
      if [ -z $rpid ]; then
         echo "$(date '+%b %e %T') - Node \"$(hostname)\": Removing Pid File $PID_FILE"
         rm -f $PID_FILE
      fi
   fi
}

# This function kills running httpd daemons of this Apache instance
#
function stop_server
{
    echo "$(date '+%b %e %T') - Node \"$(hostname)\": Stopping Apache daemons"
   
    local pid
    if [ -f $PID_FILE ]; then
        read pid < $PID_FILE
    else
        # The PID_FILE may be removed by the httpd parent process when the
        # parent process was killed. In this case, this script exits with 0.
        echo "WARNING: Function stop_server: The $PID_FILE does not exist."
        exit 0
    fi
    if [ -z $pid ]; then         
        echo "ERROR: Function stop_server: The $PID_FILE was corrupted."
	# remove the corrupted file
	remove_pidfile
        exit 1
    else
        rpid=`ps $pid | grep "$HTTP_SERVER_PROC" | awk '{print $1}'`
        if [ ! -z "$rpid" ]; then
	    $StartupCmd -f $HTTPD_CONFIG_FILE -c "PidFile $PID_FILE" -k stop	
            ret_value=$?
            check_return $ret_value 4
	fi
        ################################################################
        #Make sure that Apache process has stopped totally before exiting
        #the stop process
        if [ -f "$PID_FILE" ]; then
            sleep $APACHE_TIMEOUT
            if [ ! -f "$PID_FILE" ]; then
                return 0
            fi
            read pid < $PID_FILE
            echo "$(date '+%b %e %T') -Node \"$HOSTNAME\" WARNING: Function stop_server: The $PID_FILE still exists after apache was stopped."
            echo "$(date '+%b %e %T') -Node \"$HOSTNAME\" WARNING: Function stop_server: Forcibly stopping the apache by sending KILL signal"

            pids_left=`ps -g $pid 2>/dev/null|grep "$HTTP_SERVER_PROC" 2>/dev/null |awk '{ print $1 }'`
            kill -9 $pids_left 2>/dev/null
            sleep 1
	    ps -p $pids_left 2>/dev/null|grep -q "$HTTP_SERVER_PROC" 
            if [ $? -eq 0 ]; then
                 check_return 1 4
            fi

        fi #end [ $PID_FILE]
        #End fix
        ################################################################
	# remove the pid file if it exists and no httpd process is running
        remove_pidfile
    fi #end [ -z $pid]

}


# Check return value of functions and exit.
# Parameter:    $1: exit code of a previous command
#               $2: function ID
function check_return
{
    if (( $1 != 0 ))
    then
    case $2 in
        1)
        echo "$(date '+%b %e %T') - ERROR: Function start_server: Failed to start Apache daemons."
	echo "$(date '+%b %e %T') - Please check Apache log files for more details."
                exit 1
                ;;
        4)
                echo "$(date '+%b %e %T') - ERROR: Function stop_server: Failed to stop Apache daemons"
                exit 1
                ;;
        *)
        echo "ERROR: Failed, unknown error."
        ;;
    esac
    fi
}


# Script Start 

exit_code=0

# Test to see if we are being called to start, monitor or stop the package.

case $1 in
 start)
    start_server
 ;;

 monitor)
    echo "$(date '+%b %e %T') - Node \"$(hostname)\": Starting hahttp.mon service"
    # Monitor script parameter list:
    # hahttp.mon  [-i monitor_interval] [-r retry_times]
    . $TKIT_SCRIPT_DIR/hahttp.mon -i $MONITOR_INTERVAL -r $RETRY_TIMES
    ;;

 stop)
    stop_server
 ;;

 *)
    echo "Invalid Argument"
    ;;

esac

exit $exit_code

